@@ -1,7 +1,9 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import, print_function
+from __future__ import absolute_import, print_function, division
+
+import math
 import re
 from time import time
 import datetime
@@ -221,13 +223,16 @@ class croniter(object):
 
     def _calc(self, now, expanded, nth_weekday_of_month, is_prev):
         if is_prev:
+            now = math.ceil(now)
             nearest_diff_method = self._get_prev_nearest_diff
             sign = -1
+            offset = (len(expanded) == 6 or now % 60 > 0) and 1 or 60
         else:
+            now = math.floor(now)
             nearest_diff_method = self._get_next_nearest_diff
             sign = 1
+            offset = (len(expanded) == 6) and 1 or 60
 
-        offset = len(expanded) == 6 and 1 or 60
         dst = now = self._timestamp_to_datetime(now + sign * offset)
 
         month, year = dst.month, dst.year
@@ -503,11 +508,19 @@ class croniter(object):
                         not low or not high or int(low) > int(high)
                         or not only_int_re.search(str(step))
                     ):
-                        raise CroniterBadDateError(
-                            "[{0}] is not acceptable".format(expr_format))
+                        if i == 4 and high == '0':
+                            # handle -Sun notation -> 7
+                            high = '7'
+                        else:
+                            raise CroniterBadDateError(
+                                "[{0}] is not acceptable".format(expr_format))
 
                     low, high, step = map(int, [low, high, step])
-                    rng = range(low, high + 1, step)
+                    try:
+                        rng = range(low, high + 1, step)
+                    except ValueError as exc:
+                        raise CroniterBadCronError(
+                            'invalid range: {0}'.format(exc))
                     e_list += (["{0}#{1}".format(item, nth) for item in rng]
                         if i == 4 and nth else rng)
                 else:
